package com.gree.mall.manager.logic.fadada;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gree.mall.manager.enums.IsYesNoEnum;
import com.gree.mall.manager.plus.entity.CompanyCaAuth;
import com.gree.mall.manager.plus.service.AdminWebsitService;
import com.gree.mall.manager.plus.service.CompanyCaAuthService;
import com.gree.mall.manager.utils.fadada.Auth4FDDUtil;
import com.gree.mall.manager.utils.oss.OSSUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.Objects;

@Service
@Slf4j
@RequiredArgsConstructor
public class FDDNotifyLogic {

    private final RedisLockRegistry redisLockRegistry;
    private final OSSUtil ossUtil;
    private final CompanyCaAuthService companyCaAuthService;
    private final AdminWebsitService adminWebsitService;
    @Value("${fadada.company.auth.auto.notify.url}")
    private String fddAuthAutoNotifyUrl;

    /**
     * CA实名企业认证回调
     * @param params
     */
    public void caCompanyAuthResult(Map<String, String> params) {
        log.info("CA法大大实名企业认证回调通知结果：{}", JSONUtil.toJsonStr(params));
        if (!validCACompanyAuthSign(params)) {
            return;
        }
        this.queryCACompanyAuthResult(params.get("serialNo"));
    }

    private boolean validCACompanyAuthSign(Map<String, String> params) {
        if (CollectionUtil.isEmpty(params)) {
            return false;
        }
        String authenticationType = params.get("authenticationType");
        String certStatus = params.get("certStatus");
        String customerId = params.get("customerId");
        String serialNo = params.get("serialNo");
        String status = params.get("status");
        String statusDesc = params.get("statusDesc");
        String timestamp = params.get("timestamp");
        String sign = params.get("sign");
        String signStr = authenticationType + certStatus + customerId + serialNo + status + statusDesc;
        String newSign = Auth4FDDUtil.getMsgDigest(signStr, timestamp);
        log.info("CA法大大实名企业认证回调验签：{} {}", sign, newSign);
        return sign.equals(newSign);
    }

    public void queryCACompanyAuthResult(String transactionNo) {
        CompanyCaAuth companyCaAuth = companyCaAuthService.lambdaQuery()
                .eq(CompanyCaAuth::getFadadaTransactionNo, transactionNo)
                .one();
        if (Objects.isNull(companyCaAuth)) {
            log.error("法大大实名企业交易号不存在：" + transactionNo);
            return;
        }

        JSONObject info = Auth4FDDUtil.findCompanyCertInfo(transactionNo);
        JSONObject data = info.getJSONObject("data");
        JSONObject company = data.getJSONObject("company");
        JSONObject manager = data.getJSONObject("manager");
        String companyStatus = company.get("status", String.class);
        String managerStatus = manager.get("status", String.class);
        if (companyStatus.equals("4") && managerStatus.equals("2")) {
            String authAutoId = IdWorker.get32UUID();
            String authAutoContractId = IdWorker.getIdStr();
            String authAutoUrl = Auth4FDDUtil.beforeAuthAuto(authAutoId, authAutoContractId,
                    companyCaAuth.getFadadaCustomerId(), null, this.fddAuthAutoNotifyUrl);
            companyCaAuthService.lambdaUpdate()
                    .set(CompanyCaAuth::getFadadaIsAuth, IsYesNoEnum.YES.getKey())
                    .set(CompanyCaAuth::getFadadaStatus, companyStatus)
                    .set(CompanyCaAuth::getFadadaAuditFailReason, null)
                    .set(CompanyCaAuth::getFadadaAuthAutoId, authAutoId)
                    .set(CompanyCaAuth::getFadadaAuthAutoContractId, authAutoContractId)
                    .set(CompanyCaAuth::getFadadaAuthAutoUrl, authAutoUrl)
                    .eq(CompanyCaAuth::getCompanyCaAuthId, companyCaAuth.getCompanyCaAuthId())
                    .update();
        } else {
            String statusDesc = "";
            if (!managerStatus.equals("2")) {
                statusDesc = manager.get("auditFailReason", String.class);
                companyCaAuthService.lambdaUpdate()
                        .set(CompanyCaAuth::getFadadaIsAuth, IsYesNoEnum.NO.getKey())
                        .set(CompanyCaAuth::getFadadaStatus, "M" + managerStatus)
                        .set(CompanyCaAuth::getFadadaAuditFailReason, statusDesc)
                        .eq(CompanyCaAuth::getCompanyCaAuthId, companyCaAuth.getCompanyCaAuthId())
                        .update();
            } else {
                statusDesc = company.get("auditFailReason", String.class);
                companyCaAuthService.lambdaUpdate()
                        .set(CompanyCaAuth::getFadadaIsAuth, IsYesNoEnum.NO.getKey())
                        .set(CompanyCaAuth::getFadadaStatus, "C" + companyStatus)
                        .set(CompanyCaAuth::getFadadaAuditFailReason, statusDesc)
                        .eq(CompanyCaAuth::getCompanyCaAuthId, companyCaAuth.getCompanyCaAuthId())
                        .update();
            }
        }
    }

    public void caCompanyAuthAutoResult(Map<String, String> params) {
        log.info("CA法大大实名企业授权自动签回调通知结果：{}", JSONUtil.toJsonStr(params));
        if (!validCACompanyAuthAutoSign(params)) {
            return;
        }
        this.queryCACompanyAuthAutoResult(params);
    }

    private boolean validCACompanyAuthAutoSign(Map<String, String> params) {
        if (CollectionUtil.isEmpty(params)) {
            return false;
        }
        String transactionId = params.get("transaction_id");
        String timestamp = params.get("timestamp");
        String sign = params.get("msg_digest");
        String newSign = Auth4FDDUtil.getMsgDigest(transactionId, timestamp);
        log.info("CA法大大实名企业授权自动签回调验签：{} {}", sign, newSign);
        return sign.equals(newSign);
    }

    private void queryCACompanyAuthAutoResult(Map<String, String> params) {
        String transactionId = params.get("transaction_id");
        String contractId = params.get("contract_id");
        String viewPdfUrl = params.get("viewpdf_url");
        String downloadUrl = params.get("download_url");
        CompanyCaAuth companyCaAuth = companyCaAuthService.lambdaQuery()
                .eq(CompanyCaAuth::getFadadaAuthAutoId, transactionId)
                .eq(CompanyCaAuth::getFadadaAuthAutoContractId, contractId)
                .one();
        if (Objects.isNull(companyCaAuth)) {
            log.error("法大大企业授权自动签交易号不存在：" + transactionId);
            return;
        }

        JSONObject info = Auth4FDDUtil.queryAuthAutoStatus(companyCaAuth.getFadadaCustomerId());
        JSONObject data = info.getJSONObject("data");
        String queryTransactionNo = data.get("transactionId", String.class);
        String queryContractId = data.get("contractId", String.class);
        String queryStatus = data.get("status", String.class);
        if (queryTransactionNo.equals(transactionId) && queryContractId.equals(contractId) && queryStatus.equals("1")) {
            companyCaAuthService.lambdaUpdate()
                    .set(CompanyCaAuth::getFadadaAuthAutoStatus, IsYesNoEnum.YES.getKey())
                    .set(CompanyCaAuth::getFadadaAuthAutoViewpdfUrl, viewPdfUrl)
                    .set(CompanyCaAuth::getFadadaAuthAutoDownloadUrl, downloadUrl)
                    .eq(CompanyCaAuth::getCompanyCaAuthId, companyCaAuth.getCompanyCaAuthId())
                    .update();
        }
    }

}
